#ifndef GAMINGCORE_VULKAN_UTILS_HPP
#define GAMINGCORE_VULKAN_UTILS_HPP

#define GLFW_INCLUDE_VULKAN
#include <core/List.hpp>
#include <core/Types.hpp>
#include <core/Utility.hpp>

#include <GLFW/glfw3.h>

#include "core/VulkanBase.hpp"

struct VulkanSwapchainData {
    Core::Vulkan::Base& base;
    VkExtent2D size{};
    VkSurfaceFormatKHR surfaceFormat{};
    VkPresentModeKHR presentMode{};
    VkSharingMode sharingMode{};
    Core::List<u32> queueFamilies{};
};

bool initVulkanSwapchain(VkSwapchainKHR* sc, VulkanSwapchainData* d);
void destroyVulkanSwapchain(VkSwapchainKHR s, VkDevice d);

struct VulkanSwapchainImages {
    Core::List<VkImage> images{};
    Core::List<VkImageView> imageViews{};
};

bool initVulkanSwapchainImages(
    VulkanSwapchainImages& si, VkDevice d, VkSwapchainKHR sc, VkFormat format);
void destroyVulkanSwapchainImages(VulkanSwapchainImages& si, VkDevice d);

bool initVulkanShaderModule(VkShaderModule* sm, VkDevice d, const char* path);
void destroyVulkanShaderModule(VkShaderModule sm, VkDevice d);

bool initVulkanPipelineLayout(VkPipelineLayout* pl, VkDevice d);
void destroyVulkanPipelineLayout(VkPipelineLayout pl, VkDevice d);

bool initVulkanFramebuffers(
    Core::List<VkFramebuffer>& f, VulkanSwapchainImages& si, VkDevice d,
    VkRenderPass rp, u32 width, u32 height);
void destroyVulkanFramebuffers(Core::List<VkFramebuffer>& f, VkDevice d);

bool initCommandVulkanBuffer(VkCommandBuffer* cb, VkDevice d, VkCommandPool cp);

bool initVulkanSemaphore(VkSemaphore* s, VkDevice d);
void destroyVulkanSemaphore(VkSemaphore s, VkDevice d);

bool initVulkanFence(VkFence* f, VkDevice d);
void destroyVulkanFence(VkFence f, VkDevice d);

#endif
